package org.netkernel.lang.javascript;

import org.mozilla.javascript.Context;
import org.mozilla.javascript.ContextFactory;
import org.mozilla.javascript.ImporterTopLevel;
import org.mozilla.javascript.RhinoException;
import org.mozilla.javascript.Script;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.ScriptableObject;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFRequestReadOnly;
import org.netkernel.layer0.nkf.impl.NKFContextImpl;
import org.netkernel.layer0.util.RequestScopeClassLoader;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.urii.impl.NetKernelException;

public class JavascriptRuntime extends StandardAccessorImpl
{
	public JavascriptRuntime()
	{	this.declareThreadSafe();
	}

	@Override
	public void onRequest(INKFRequestContext context) throws Exception
	{	if (context.getThisRequest().getVerb()==INKFRequestReadOnly.VERB_META)
		{	super.onMeta(context);
		}
		else
		{	String sourceIdentifier=context.getThisRequest().getArgumentValue("operator");
			((NKFContextImpl)context).setRuntimeSourceIdentifier(sourceIdentifier);
			try
			{	ClassLoader CL=new RequestScopeClassLoader(context.getKernelContext().getThisKernelRequest().getRequestScope());
				Context ctx=ContextFactory.getGlobal().enterContext();
				ctx.setApplicationClassLoader(CL);
				Scriptable scope =new ImporterTopLevel(ctx);
				Object wrappedContext = Context.javaToJS(context, scope);
				ScriptableObject.putProperty(scope, "context", wrappedContext);
				Object wrappedLoader = Context.javaToJS(new LibraryLoader(context, ctx, scope), scope);
				ScriptableObject.putProperty(scope, "contextloader", wrappedLoader);
				Script s=context.source("arg:operator", Script.class);
				
				s.exec(ctx, scope);
			}
			catch(RhinoException e)
			{	NetKernelException nke=new NetKernelException("JavascriptException", "line: "+e.lineNumber()+"  "+e.getMessage(), e);
				throw nke;
			}
			finally
			{	Context.exit();
			}
		}
	}
	
	

}
